package goaml.lang.state;

import org.w3c.dom.Node;

import goaml.lang.entity.Entity;

public abstract class AbstractState {

	protected String name;
	protected int numCohort;
	protected AbstractState followingCohort;

	protected AbstractState() {
	}

	// Parse ONE tag
	public static final AbstractState parse(final Entity entity, final Node node,
			final boolean canReceiveInitialValue) {
		if (node == null) {
			throw new RuntimeException("Node is null");
		}

		AbstractState state;

		switch (node.getNodeName()) {
		case "CompositeState":
			state = new CompositeState(entity, node, canReceiveInitialValue);
			break;

		case "SequentialState":
			state = new SequentialState(entity, node, canReceiveInitialValue);
			break;

		case "State":
			state = new State(node, canReceiveInitialValue);
			break;

		default:
			throw new RuntimeException(node.getNodeName() + " isn't a valid node name");
		}

		// Add the state to the list of states of the entity
		entity.addState(state);

		// Return the state created. Useless in stateChart, because the state already register
		// itself to the entity : "entity.addState(state)"
		// But use in CompositeState and SequentialState to fill their own list of subStates
		return state;
	}

	public final String getName() {
		return this.name;
	}

	public final int getNumCohort() {
		return this.numCohort;
	}

	public final void setNumCohort(final int numCohort) {
		this.numCohort = numCohort;
	}

	public final AbstractState getFollowingCohort() {
		// If followingCohort has never been initialized,
		// by default the state itself is his following cohort
		// Useful in IL2Prolif for example, when the last cohort can proliferate
		// but it increase itself, not another cohort
		if (this.followingCohort == null) {
			this.followingCohort = this;
		}
		return this.followingCohort;
	}

	public final void setFollowingCohort(final AbstractState state) {
		this.followingCohort = state;
	}

	public abstract double getValue();

}
