
package goaml.lang.state;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

public class State extends AbstractState {

	private double value;

	protected State(final Node node, final boolean canReceiveInitialValue) {
		if (node == null) {
			throw new RuntimeException("Node is null");
		}
		NamedNodeMap map = node.getAttributes();

		// Required attributes
		if (map.getNamedItem("name") != null) {
			name = map.getNamedItem("name").getTextContent();
		} else {
			throw new RuntimeException("\"name\" attribute missed");
		}

		// Optional attributes
		if (map.getNamedItem("value") != null) {
			if (canReceiveInitialValue) {
				// Initialize value only if canReceiveInitialValue == true
				if (Double.valueOf(map.getNamedItem("value").getTextContent()) >= 0) {
					value = Double.valueOf(map.getNamedItem("value").getTextContent());
				} else {
					throw new RuntimeException("\"value\" is negative in " + node.getNodeName() + " tag");
				}
			}
		}
	}

	@Override
	public final double getValue() {
		return this.value;
	}

	public final void add(final double rate) {
		this.value += rate;
	}

	public final void substract(final double rate) {
		this.value -= rate;
		if (this.value < 0) {
			this.value = 0;
		}
	}

	public final void reset() {
		this.value = 0;
	}

}
