package goaml.lang.transition;

import java.util.ArrayList;
import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.entity.Entity;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.AbstractState;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class GCVProliferation extends AbstractTransition {

	private Constant proliferationRate;
	private Constant Y;
	private List<AbstractState> population;

	public GCVProliferation() {
	}

	private GCVProliferation(final State source, final State target, final Constant proliferationRate, final Constant Y,
			final List<AbstractState> population) {
		super(source, target);
		this.proliferationRate = proliferationRate;
		this.Y = Y;
		this.population = population;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		proliferationRate = Constant.getConstant("proliferationRate", constants, source, target);
		Y = Constant.getConstant("Y", constants, source, target);

		// Check if all attributes have been initialized
		if (proliferationRate == null && Y == null) {
			return;
		}

		population = new ArrayList<AbstractState>();

		List<Entity> entities = stateChart.getEntities();
		for (Entity entity : entities) {
			population.addAll(entity.getStatesByName(source.getName()));
		}

		stateChart.addTransition(
				new GCVProliferation(source, (State) source.getFollowingCohort(), proliferationRate, Y, population));
	}

	@Override
	public void compute(StateChart stateChart) {
		double rate = this.proliferationRate.getValue() * this.source.getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		if(Y.getValue() > 0) {
			stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
			stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, 2 * rate));
		}
	}

}
