package goaml.lang.transition;

import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class Input extends AbstractTransition {

	private Constant input;

	public Input() {
	}

	private Input(final State source, final State target, final Constant input) {
		super(source, target);
		this.input = input;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		input = Constant.getConstant("inputRate", constants, source, target);

		// Check if all attributes have been initialized
		if (input == null) {
			return;
		}

		stateChart.addTransition(new Input(source, target, input));

	}

	@Override
	public void compute(StateChart stateChart) {
		// Compute the rate
		double rate = this.input.getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, 0));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, rate));
	}

}
